@@ -57,6 +57,15 @@ extern void _clear_last_dumped();
  */
 extern PyObject *_get_referents(PyObject *c_obj);
 
+/**
+ * Return a (mutable) dict of known special cases.
+ * 
+ * These are objects whose size is not reported properly, but which we have
+ * figured out via trial-and-error.
+ * The key is tp_name strings, the value is a PyInt of the appropriate size.
+ */
+extern PyObject *_get_special_case_dict();
+
 
 #endif // _SCANNER_CORE_H_
 
