@@ -1,14 +1,14 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
-# 
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
 # published by the Free Software Foundation.
-# 
+#
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -18,6 +18,7 @@ import gc
 import sys
 import tempfile
 import types
+import zlib
 
 from meliae import (
     _scanner,
@@ -27,6 +28,10 @@ from meliae import (
 
 STRING_BASE = 8
 STRING_SCALING = 4
+if sys.version_info[:2] >= (2, 7):
+    # In python2.7 they 'shrunk' strings by a couple bytes, by changing where
+    # the pointer was. So their base size is a few bytes smaller
+    STRING_BASE = 5
 
 
 class TestSizeOf(tests.TestCase):
@@ -38,6 +43,7 @@ class TestSizeOf(tests.TestCase):
         self.assertEqual(expected_size, _scanner.size_of(obj))
 
     def test_empty_string(self):
+        fixed_size = STRING_BASE
         self.assertSizeOf(STRING_SCALING, '', extra_size=0+STRING_BASE, has_gc=False)
 
     def test_short_string(self):
@@ -170,6 +176,60 @@ class TestSizeOf(tests.TestCase):
         # back to the original size
         self.assertSizeOf(4, CustomSize(-1), has_gc=True)
 
+    def test_size_of_special(self):
+        class CustomWithoutSizeof(object):
+            pass
+        log = []
+        def _size_32(obj):
+            log.append(obj)
+            return 800
+        def _size_64(obj):
+            log.append(obj)
+            return 1600
+            
+        obj = CustomWithoutSizeof()
+        self.assertSizeOf(4, obj)
+        _scanner.add_special_size('CustomWithoutSizeof', _size_32, _size_64)
+        try:
+            self.assertSizeOf(200, obj)
+        finally:
+            _scanner.add_special_size('CustomWithoutSizeof', None, None)
+        self.assertEqual([obj], log)
+        del log[:]
+        self.assertSizeOf(4, obj)
+        self.assertEqual([], log)
+
+    def test_size_of_special_neg1(self):
+        # Returning -1 falls back to the regular __sizeof__, etc interface
+        class CustomWithoutSizeof(object):
+            pass
+        log = []
+        def _size_neg1(obj):
+            log.append(obj)
+            return -1
+        obj = CustomWithoutSizeof()
+        self.assertSizeOf(4, obj)
+        _scanner.add_special_size('CustomWithoutSizeof', _size_neg1, _size_neg1)
+        try:
+            self.assertSizeOf(4, obj)
+        finally:
+            _scanner.add_special_size('CustomWithoutSizeof', None, None)
+        self.assertEqual([obj], log)
+
+    def test_size_of_zlib_compress_obj(self):
+        # zlib compress objects allocate a lot of extra buffers, we want to
+        # track that. Note that we are approximating it, because we don't
+        # actually inspect the C attributes. But it is a closer approximation
+        # than not doing this.
+        c = zlib.compressobj()
+        self.assertTrue(_scanner.size_of(c) > 256000)
+        self.assertEqual(0, _scanner.size_of(c) % _scanner._word_size)
+
+    def test_size_of_zlib_decompress_obj(self):
+        d = zlib.decompressobj()
+        self.assertTrue(_scanner.size_of(d) > 30000)
+        self.assertEqual(0, _scanner.size_of(d) % _scanner._word_size)
+
 
 def _string_to_json(s):
     out = ['"']
